@@ -1,22 +1,20 @@
-# -*- coding: utf-8 -*-
 """Module that provides functionality for content manipulation."""
 
 from copy import copy as _copy
-from pkg_resources import DistributionNotFound
-from pkg_resources import get_distribution
-from pkg_resources import parse_version
 from plone.api import portal
 from plone.api.exc import InvalidParameterError
 from plone.api.validation import at_least_one_of
 from plone.api.validation import mutually_exclusive_parameters
 from plone.api.validation import required_parameters
-from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException  # noqa
+from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException
 from plone.app.uuid.utils import uuidToObject
 from plone.uuid.interfaces import IUUID
+from Products.CMFCore.DynamicType import DynamicType
 from Products.CMFCore.WorkflowCore import WorkflowException
 from zope.component import getMultiAdapter
 from zope.component import getSiteManager
 from zope.container.interfaces import INameChooser
+from zope.globalrequest import getRequest
 from zope.interface import Interface
 from zope.interface import providedBy
 
@@ -24,33 +22,18 @@ import random
 import transaction
 
 
-try:
-    get_distribution('Products.Archetypes')
-except DistributionNotFound:
-    class IBaseObject(Interface):
-        """Fake Products.Archetypes.interfaces.base.IBaseObject"""
-else:
-    from Products.Archetypes.interfaces.base import IBaseObject
-
-# Old linkintegrity (Plone <= 5.0b4) or new (Plone > 5.0b4)
-linkintegrity_version = get_distribution('plone.app.linkintegrity').version
-if parse_version(linkintegrity_version) >= parse_version('3.0.dev0'):
-    NEW_LINKINTEGRITY = True
-else:
-    NEW_LINKINTEGRITY = False
-
 _marker = []
 
 
-@required_parameters('container', 'type')
-@at_least_one_of('id', 'title')
+@required_parameters("container", "type")
+@at_least_one_of("id", "title")
 def create(
     container=None,
     type=None,
     id=None,
     title=None,
     safe_id=False,
-    **kwargs  # NOQA: C816
+    **kwargs  # NOQA: C816, S101
 ):
     """Create a new content item.
 
@@ -71,18 +54,19 @@ def create(
         conflicting with another object in the target container, raise an
         InvalidParameterError. When True, choose a new, non-conflicting id.
     :type safe_id: boolean
+
     :returns: Content object
     :raises:
         KeyError,
         :class:`~plone.api.exc.MissingParameterError`,
         :class:`~plone.api.exc.InvalidParameterError`
-    :Example: :ref:`content_create_example`
+    :Example: :ref:`content-create-example`
     """
     # Create a temporary id if the id is not given
     content_id = not safe_id and id or str(random.randint(0, 99999999))
 
     if title:
-        kwargs['title'] = title
+        kwargs["title"] = title
 
     try:
         container.invokeFactory(type, content_id, **kwargs)
@@ -95,26 +79,16 @@ def create(
 
         raise InvalidParameterError(
             "Cannot add a '{obj_type}' object to the container.\n"
-            'Allowed types are:\n'
-            '{allowed_types}\n'
-            '{message}'.format(
+            "Allowed types are:\n"
+            "{allowed_types}\n"
+            "{message}".format(
                 obj_type=type,
-                allowed_types='\n'.join(sorted(types)),
+                allowed_types="\n".join(sorted(types)),
                 message=str(e),
             ),
         )
 
     content = container[content_id]
-
-    # Archetypes specific code
-    if IBaseObject.providedBy(content):
-        # Will finish Archetypes content item creation process,
-        # rename-after-creation and such
-        # Passing values as a dict with None values so values set by
-        # invokeFactory don't get overridden.
-        # '': '' is required so that bool(values) is True.
-        content.processForm(values={'': ''})
-
     if not id or (safe_id and id):
         # Create a new id from title
         chooser = INameChooser(container)
@@ -132,8 +106,8 @@ def create(
     return content
 
 
-@mutually_exclusive_parameters('path', 'UID')
-@at_least_one_of('path', 'UID')
+@mutually_exclusive_parameters("path", "UID")
+@at_least_one_of("path", "UID")
 def get(path=None, UID=None):
     """Get an object.
 
@@ -145,28 +119,31 @@ def get(path=None, UID=None):
     :returns: Content object
     :raises:
         ValueError,
-    :Example: :ref:`content_get_example`
+    :Example: :ref:`content-get-example`
     """
     if path:
         site = portal.get()
-        site_absolute_path = '/'.join(site.getPhysicalPath())
-        if not path.startswith('{path}'.format(path=site_absolute_path)):
-            path = '{site_path}{relative_path}'.format(
+        site_absolute_path = "/".join(site.getPhysicalPath())
+        if not path.startswith("{path}".format(path=site_absolute_path)):
+            path = "{site_path}{relative_path}".format(
                 site_path=site_absolute_path,
                 relative_path=path,
             )
-
         try:
-            return site.restrictedTraverse(path)
+            content = site.restrictedTraverse(path)
         except (KeyError, AttributeError):
             return None  # When no object is found don't raise an error
+        else:
+            # Only return a content if it implements DynamicType,
+            # which is true for Dexterity content and Comment (plone.app.discussion)
+            return content if isinstance(content, DynamicType) else None
 
     elif UID:
         return uuidToObject(UID)
 
 
-@required_parameters('source')
-@at_least_one_of('target', 'id')
+@required_parameters("source")
+@at_least_one_of("target", "id")
 def move(source=None, target=None, id=None, safe_id=False):
     """Move the object to the target container.
 
@@ -175,7 +152,7 @@ def move(source=None, target=None, id=None, safe_id=False):
     :param target: Target container to which the source object will
         be moved. If no target is specified, the source object's container will
         be used as a target, effectively making this operation a rename
-        (:ref:`content_rename_example`).
+        (:ref:`content-rename-example`).
     :type target: Folderish content object
     :param id: Pass this parameter if you want to change the id of the moved
         object on the target location. If the new id conflicts with another
@@ -190,7 +167,7 @@ def move(source=None, target=None, id=None, safe_id=False):
     :raises:
         KeyError
         ValueError
-    :Example: :ref:`content_move_example`
+    :Example: :ref:`content-move-example`
     """
     source_id = source.getId()
 
@@ -208,7 +185,7 @@ def move(source=None, target=None, id=None, safe_id=False):
         return target[source_id]
 
 
-@required_parameters('obj', 'new_id')
+@required_parameters("obj", "new_id")
 def rename(obj=None, new_id=None, safe_id=False):
     """Rename the object.
 
@@ -221,7 +198,7 @@ def rename(obj=None, new_id=None, safe_id=False):
         InvalidParameterError. When True, choose a new, non-conflicting id.
     :type safe_id: boolean
     :returns: Content object that was renamed
-    :Example: :ref:`content_rename_example`
+    :Example: :ref:`content-rename-example`
     """
     obj_id = obj.getId()
     container = obj.aq_parent
@@ -235,8 +212,8 @@ def rename(obj=None, new_id=None, safe_id=False):
     return container[new_id]
 
 
-@required_parameters('source')
-@at_least_one_of('target', 'id')
+@required_parameters("source")
+@at_least_one_of("target", "id")
 def copy(source=None, target=None, id=None, safe_id=False):
     """Copy the object to the target container.
 
@@ -259,7 +236,7 @@ def copy(source=None, target=None, id=None, safe_id=False):
     :raises:
         KeyError,
         ValueError
-    :Example: :ref:`content_copy_example`
+    :Example: :ref:`content-copy-example`
     """
     source_id = source.getId()
 
@@ -270,7 +247,7 @@ def copy(source=None, target=None, id=None, safe_id=False):
         source.aq_parent.manage_copyObjects(source_id),
     )
 
-    new_id = copy_info[0]['new_id']
+    new_id = copy_info[0]["new_id"]
     if id:
         if not safe_id and id in target:
             msg = "Duplicate ID '{0}' in '{1}' for '{2}'"
@@ -281,8 +258,8 @@ def copy(source=None, target=None, id=None, safe_id=False):
         return target[new_id]
 
 
-@mutually_exclusive_parameters('obj', 'objects')
-@at_least_one_of('obj', 'objects')
+@mutually_exclusive_parameters("obj", "objects")
+@at_least_one_of("obj", "objects")
 def delete(obj=None, objects=None, check_linkintegrity=True):
     """Delete the object(s).
 
@@ -298,7 +275,7 @@ def delete(obj=None, objects=None, check_linkintegrity=True):
         ValueError
         plone.app.linkintegrity.exceptions.LinkIntegrityNotificationException
 
-    :Example: :ref:`content_delete_example`
+    :Example: :ref:`content-delete-example`
     """
     objects = [obj] if obj else objects
 
@@ -306,35 +283,25 @@ def delete(obj=None, objects=None, check_linkintegrity=True):
     if not objects:
         return
 
-    if check_linkintegrity and NEW_LINKINTEGRITY:
+    if check_linkintegrity:
         site = portal.get()
         linkintegrity_view = get_view(
-            name='delete_confirmation_info',
+            name="delete_confirmation_info",
             context=site,
             request=site.REQUEST,
         )
-
         # look for breaches and manually raise a exception
         breaches = linkintegrity_view.get_breaches(objects)
         if breaches:
             raise LinkIntegrityNotificationException(
-                'Linkintegrity-breaches: {0}'.format(breaches),
+                "Linkintegrity-breaches: {}".format(breaches),
             )
 
     for obj_ in objects:
-        if not check_linkintegrity and not NEW_LINKINTEGRITY:
-            # old style ignoring breaches:
-            # we have to explicitly ignore the exception
-            try:
-                obj_.aq_parent.manage_delObjects([obj_.getId()])
-            except LinkIntegrityNotificationException:
-                pass
-        else:
-            # All other cases
-            obj_.aq_parent.manage_delObjects([obj_.getId()])
+        obj_.aq_parent.manage_delObjects([obj_.getId()])
 
 
-@required_parameters('obj')
+@required_parameters("obj")
 def get_state(obj=None, default=_marker):
     """Get the current workflow state of the object.
 
@@ -345,16 +312,16 @@ def get_state(obj=None, default=_marker):
     :rtype: string
     :raises:
         Products.CMFCore.WorkflowCore.WorkflowException
-    :Example: :ref:`content_get_state_example`
+    :Example: :ref:`content-get-state-example`
     """
-    workflow = portal.get_tool('portal_workflow')
+    workflow = portal.get_tool("portal_workflow")
 
     if default is not _marker and not workflow.getWorkflowsFor(obj):
         return default
 
     # This still raises WorkflowException when the workflow state is broken,
     # ie 'review_state' is absent
-    return workflow.getInfoFor(ob=obj, name='review_state')
+    return workflow.getInfoFor(ob=obj, name="review_state")
 
 
 # work backwards from our end state
@@ -390,7 +357,8 @@ def _find_path(maps, path, current_state, start_state):
 
 
 def _wf_transitions_for(workflow, from_state, to_state):
-    """Get a list of transition IDs required to transition
+    """Get list of transition IDs required to transition.
+
     from ``from_state`` to ``to_state``.
 
     :param workflow: Workflow object which contains states and transitions
@@ -431,14 +399,14 @@ def _transition_to(obj, workflow, to_state, **kwargs):
     # via any route we can find
     for wf in workflow.getWorkflowsFor(obj):
         status = workflow.getStatusOf(wf.getId(), obj)
-        if not status or not status.get('review_state'):
+        if not status or not status.get("review_state"):
             continue
-        if status['review_state'] == to_state:
+        if status["review_state"] == to_state:
             return
 
         transitions = _wf_transitions_for(
             wf,
-            status['review_state'],
+            status["review_state"],
             to_state,
         )
         if not transitions:
@@ -457,11 +425,13 @@ def _transition_to(obj, workflow, to_state, **kwargs):
         break
 
 
-@required_parameters('obj')
-@at_least_one_of('transition', 'to_state')
-@mutually_exclusive_parameters('transition', 'to_state')
+@required_parameters("obj")
+@at_least_one_of("transition", "to_state")
+@mutually_exclusive_parameters("transition", "to_state")
 def transition(obj=None, transition=None, to_state=None, **kwargs):
-    """Perform a workflow transition for the object or attempt to perform
+    """Perform a workflow transition.
+
+    for the object or attempt to perform
     workflow transitions on the object to reach the given state.
     The later will not guarantee that transition guards conditions can be met.
 
@@ -477,60 +447,60 @@ def transition(obj=None, transition=None, to_state=None, **kwargs):
     :raises:
         :class:`~plone.api.exc.MissingParameterError`,
         :class:`~plone.api.exc.InvalidParameterError`
-    :Example: :ref:`content_transition_example`
+    :Example: :ref:`content-transition-example`
     """
-    workflow = portal.get_tool('portal_workflow')
+    workflow = portal.get_tool("portal_workflow")
     if transition is not None:
         try:
             workflow.doActionFor(obj, transition, **kwargs)
         except WorkflowException:
-            transitions = [
-                action['id'] for action in workflow.listActions(object=obj)
-            ]
+            transitions = [action["id"] for action in workflow.listActions(object=obj)]
 
             raise InvalidParameterError(
-                "Invalid transition '{0}'.\n"
-                'Valid transitions are:\n'
-                '{1}'.format(transition, '\n'.join(sorted(transitions))),
+                "Invalid transition '{}'.\n"
+                "Valid transitions are:\n"
+                "{}".format(transition, "\n".join(sorted(transitions))),
             )
     else:
         _transition_to(obj, workflow, to_state, **kwargs)
-        if workflow.getInfoFor(obj, 'review_state') != to_state:
+        if workflow.getInfoFor(obj, "review_state") != to_state:
             raise InvalidParameterError(
-                'Could not find workflow to set state to {0} on {1}'.format(
+                "Could not find workflow to set state to {} on {}".format(
                     to_state,
                     obj,
                 ),
             )
 
 
-@required_parameters('obj')
+@required_parameters("obj")
 def disable_roles_acquisition(obj=None):
     """Disable acquisition of local roles on given obj.
+
     Set __ac_local_roles_block__ = 1 on obj.
 
     :param obj: [required] Context object to block the acquisition on.
     :type obj: Content object
-    :Example: :ref:`content_disable_roles_acquisition_example`
+    :Example: :ref:`content-disable-roles-acquisition-example`
     """
-    plone_utils = portal.get_tool('plone_utils')
+    plone_utils = portal.get_tool("plone_utils")
     plone_utils.acquireLocalRoles(obj, status=0)
 
 
-@required_parameters('obj')
+@required_parameters("obj")
 def enable_roles_acquisition(obj=None):
     """Enable acquisition of local roles on given obj.
+
     Set __ac_local_roles_block__ = 0 on obj.
 
     :param obj: [required] Context object to enable the acquisition on.
     :type obj: Content object
-    :Example: :ref:`content_enable_roles_acquisition_example`
+    :Example: :ref:`content-enable-roles-acquisition-example`
     """
-    plone_utils = portal.get_tool('plone_utils')
+    plone_utils = portal.get_tool("plone_utils")
     plone_utils.acquireLocalRoles(obj, status=1)
 
 
-@required_parameters('name', 'context', 'request')
+@required_parameters("name", "context")
 def get_view(name=None, context=None, request=None):
     """Get a BrowserView object.
 
@@ -543,8 +513,10 @@ def get_view(name=None, context=None, request=None):
     :raises:
         :class:`~plone.api.exc.MissingParameterError`,
         :class:`~plone.api.exc.InvalidParameterError`
-    :Example: :ref:`content_get_view_example`
+    :Example: :ref:`content-get-view-example`
     """
+    if request is None:
+        request = getRequest()
     # We do not use exceptionhandling to detect if the requested view is
     # available, because the __init__ of said view will contain
     # errors in client code.
@@ -562,16 +534,16 @@ def get_view(name=None, context=None, request=None):
     if name not in available_view_names:
         raise InvalidParameterError(
             "Cannot find a view with name '{name}'.\n"
-            'Available views are:\n'
-            '{views}'.format(
+            "Available views are:\n"
+            "{views}".format(
                 name=name,
-                views='\n'.join(sorted(available_view_names)),
+                views="\n".join(sorted(available_view_names)),
             ),
         )
     return getMultiAdapter((context, request), name=name)
 
 
-@required_parameters('obj')
+@required_parameters("obj")
 def get_uuid(obj=None):
     """Get the object's Universally Unique IDentifier (UUID).
 
@@ -581,7 +553,7 @@ def get_uuid(obj=None):
     :rtype: string
     :raises:
         ValueError
-    :Example: :ref:`content_get_uuid_example`
+    :Example: :ref:`content-get-uuid-example`
     """
     return IUUID(obj)
 
@@ -594,18 +566,35 @@ def _parse_object_provides_query(query):
         query for multiple values
         (eg. `{'query': [Iface1, Iface2], 'operator': 'or'}`)
     """
-    operator = 'or'
     ifaces = query
-    if isinstance(query, dict):
-        operator = query.get('operator', operator)
-        ifaces = query.get('query', [])
-    elif not isinstance(query, (list, tuple)):
-        ifaces = [query]
+    operator = "or"
+    query_not = []
 
-    return {
-        'query': [getattr(x, '__identifier__', x) for x in ifaces],
-        'operator': operator,
-    }
+    if isinstance(query, dict):
+        ifaces = query.get("query", [])
+        operator = query.get("operator", operator)
+        query_not = query.get("not", [])
+        # KeywordIndex also supports "range",
+        # but that's not useful for querying object_provides
+
+    if not isinstance(ifaces, (list, tuple)):
+        ifaces = [ifaces]
+    ifaces = [getattr(x, "__identifier__", x) for x in ifaces]
+
+    if not isinstance(query_not, (list, tuple)):
+        query_not = [query_not]
+    query_not = [getattr(x, "__identifier__", x) for x in query_not]
+
+    result = {}
+
+    if ifaces:
+        result["query"] = ifaces
+        result["operator"] = operator
+
+    if query_not:
+        result["not"] = query_not
+
+    return result
 
 
 def find(context=None, depth=None, **kwargs):
@@ -617,72 +606,23 @@ def find(context=None, depth=None, **kwargs):
     :type obj: Content object
     :returns: Catalog brains
     :rtype: List
-    :Example: :ref:`content_find_example`
+    :Example: :ref:`content-find-example`
 
-    Find works alike catalog(). Indexes are passing in as arguments with the
-    search query as the values.
-
-    Specify indexes as arguments:
-    >>> find(portal_type='Document')
-
-    or combinations of indexes.
-    >>> find(portal_type='Document', SearchableText='Team')
-
-
-    Differences to using the catalog directly are:
-
-    The context argument allows passing in an context object, instead
-    of path='/'.join(context.getPhysicalPath().
-
-    >>> find(context=context)
-    - or -
-    >>> find(context=context, portal_type='Document')
-
-    Specifing the search depth is supported using the `depth` argument.
-    >>> find(depth=1)
-
-    Using `depth` needs a context for it's path. If no context and no
-    path is passed, the portal root is used.
-    >>> find(context=portal, depth=1, portal_type='Document')
-    - or -
-    >>> find(depth=1, path='/plone/folder', portal_type='Document')
-    - or -
-    >>> find(depth=1, portal_type='Document')
-
-    The path can be queried directly, too:
-    >>> find(path={'query': '/plone/about/team', 'depth': 1})
-
-    The `object_provides` index/argument allows Interface objects as well as
-    identifiers. It also supports querying multiple interfaces combined with
-    `and` or `or`.
-    >>> find(object_provides=IATDocument)
-    - or -
-    >>> find(object_provides=IATDocument.__identifier__)
-    - or -
-    >>> find(object_provides={
-    ...     'query': [IATFolder, INavigationRoot],
-    ...     'operator': 'and',
-    ... })
-
-
-    An empty resultset is returned if no valid indexes are queried.
-    >>> len(find())
-    >>> 0
     """
     query = {}
     query.update(**kwargs)
 
     # Save the original path to maybe restore it later.
-    orig_path = query.get('path')
+    orig_path = query.get("path")
     if isinstance(orig_path, dict):
-        orig_path = orig_path.get('query')
+        orig_path = orig_path.get("query")
 
     # Passing a context or depth overrides the existing path query,
     # for now.
     if context or depth is not None:
         # Make the path a dictionary, unless it already is.
         if not isinstance(orig_path, dict):
-            query['path'] = {}
+            query["path"] = {}
 
     # Limit search depth
     if depth is not None:
@@ -691,20 +631,20 @@ def find(context=None, depth=None, **kwargs):
             context = portal.get()
         else:
             # Restore the original path
-            query['path']['query'] = orig_path
-        query['path']['depth'] = depth
+            query["path"]["query"] = orig_path
+        query["path"]["depth"] = depth
 
     if context is not None:
-        query['path']['query'] = '/'.join(context.getPhysicalPath())
+        query["path"]["query"] = "/".join(context.getPhysicalPath())
 
     # Convert interfaces to their identifiers and also allow to query
     # multiple values using {'query:[], 'operator':'and|or'}
-    obj_provides = query.get('object_provides', [])
+    obj_provides = query.get("object_provides", [])
     if obj_provides:
-        query['object_provides'] = _parse_object_provides_query(obj_provides)
+        query["object_provides"] = _parse_object_provides_query(obj_provides)
 
     # Make sure we don't dump the whole catalog.
-    catalog = portal.get_tool('portal_catalog')
+    catalog = portal.get_tool("portal_catalog")
     indexes = catalog.indexes()
     valid_indexes = [index for index in query if index in indexes]
     if not valid_indexes:
