@@ -170,8 +170,11 @@ def create(source=None, target=None, relationship=None):
         "from_id": from_id,
         "to_id": to_id,
     }
-    for rel in relation_catalog.findRelations(query):
-        relation_catalog.unindex(rel)
+    has_relation = (
+        False
+        if (len([el for el in relation_catalog.findRelations(query)]) == 0)
+        else True
+    )
 
     if from_attribute == referencedRelationship:
         # Don't mess with linkintegrity-relations!
@@ -212,10 +215,11 @@ def create(source=None, target=None, relationship=None):
             source.absolute_url(),
             target.absolute_url(),
         )
-        existing_relations = getattr(source, from_attribute, [])
-        existing_relations.append(RelationValue(to_id))
-        setattr(source, from_attribute, existing_relations)
-        modified(source)
+        if not has_relation:
+            existing_relations = getattr(source, from_attribute, [])
+            existing_relations.append(RelationValue(to_id))
+            setattr(source, from_attribute, existing_relations)
+            modified(source)
         return
 
     elif isinstance(field, (Relation, RelationChoice)):
@@ -225,8 +229,9 @@ def create(source=None, target=None, relationship=None):
             source.absolute_url(),
             target.absolute_url(),
         )
-        setattr(source, from_attribute, RelationValue(to_id))
-        modified(source)
+        if not has_relation:
+            setattr(source, from_attribute, RelationValue(to_id))
+            modified(source)
         return
 
     # If we end up here, someone is making a relationship that
